@@ -1,3 +1,4 @@
+/* $Cambridge: exim/exim-src/exim_monitor/em_TextPop.c,v 1.1 2004/10/07 10:39:01 ph10 Exp $ */
 /* $XConsortium: TextPop.c,v 1.22 91/07/25 18:10:22 rws Exp $ */
 
 /***********************************************************
@@ -32,6 +33,9 @@ SOFTWARE.
 * some systems that declare it differently. September 1996.                 *
 * Added the arguments into the static functions declared at the head, to    *
 * stop some compiler warnings. August 1999.                                 *
+* Took out the separate declarations of errno and sys_nerr at the start,    *
+* because they too aren't actually used, and the declaration causes trouble *
+* on some systems. December 2002.                                           *
 ****************************************************************************/
 
 
@@ -71,7 +75,7 @@ SOFTWARE.
 #include <X11/Xos.h>		/* for O_RDONLY */
 #include <errno.h>
 
-extern int errno, sys_nerr;
+/* extern int errno, sys_nerr; */
 /* extern char* sys_errlist[]; */
 
 #define DISMISS_NAME  ("cancel")
