@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/exim_monitor/em_hdr.h,v 1.6 2007/01/08 10:50:17 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/exim_monitor/em_hdr.h,v 1.8 2009/11/16 19:50:36 nm4 Exp $ */
 
 /*************************************************
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2007 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -87,7 +87,7 @@ anything. */
 
 /* Regular expression include */
 
-#include "pcre/pcre.h"
+#include <pcre.h>
 
 /* Includes from the main source of Exim. We need to have MAXPACKET defined for
 the benefit of structs.h. One of these days I should tidy up this interface so
