@@ -1,4 +1,5 @@
 #! /bin/sh
+# $Cambridge: exim/exim-src/scripts/Configure-os.h,v 1.3 2008/04/13 09:22:45 tom Exp $
 
 # Shell script to create a link to the appropriate OS-specific header file.
 
@@ -27,7 +28,10 @@ then    echo ""
 	exit 1;
 fi
 rm -f os.h
-ln -s ../OS/os.h-$os os.h || exit 1
+
+# In order to accomodate for the fudge below, copy the file instead of
+# symlinking it. Otherwise we pollute the clean copy with the fudge.
+cp -p ../OS/os.h-$os os.h || exit 1
 
 # Special-purpose fudge for older versions of Linux (pre 2.1.15) that
 # use the structure name "options" instead of "ip_options".
