@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/buildconfig.c,v 1.13 2007/01/08 10:50:17 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/buildconfig.c,v 1.15 2009/11/16 19:50:36 nm4 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2007 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -680,7 +680,8 @@ while (fgets(buffer, sizeof(buffer), base) != NULL)
     {
     char *wcs = getenv("WITH_CONTENT_SCAN");
     char *wod = getenv("WITH_OLD_DEMIME");
-    if (wcs != NULL || wod != NULL)
+    char *dcc = getenv("EXPERIMENTAL_DCC");
+    if (wcs != NULL || wod != NULL || dcc != NULL)
       fprintf(new, "#define WITH_CONTENT_SCAN     yes\n");
     else fprintf(new, "/* WITH_CONTENT_SCAN not set */\n");
     continue;
