@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/mime.c,v 1.14 2006/02/22 14:46:44 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/mime.c,v 1.19 2009/11/11 10:08:02 nm4 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -41,195 +41,189 @@ void mime_set_anomaly(int level, char *text) {
            0-255 - char to write
 */
 
-unsigned int mime_qp_hstr_i(uschar *cptr) {
-  unsigned int i, j = 0;
-  while (cptr && *cptr && isxdigit(*cptr)) {
-    i = *cptr++ - '0';
-    if (9 < i) i -= 7;
-    j <<= 4;
-    j |= (i & 0x0f);
-  }
-  return(j);
-}
-
-uschar *mime_decode_qp_char(uschar *qp_p,int *c) {
-  uschar hex[] = {0,0,0};
-  int nan = 0;
+uschar *mime_decode_qp_char(uschar *qp_p, int *c) {
   uschar *initial_pos = qp_p;
 
   /* advance one char */
   qp_p++;
 
-  REPEAT_FIRST:
-  if ( (*qp_p == '\t') || (*qp_p == ' ') || (*qp_p == '\r') )  {
-    /* tab or whitespace may follow
-       just ignore it, but remember
-       that this is not a valid hex
-       encoding any more */
-    nan = 1;
-    qp_p++;
-    goto REPEAT_FIRST;
-  }
-  else if ( (('0' <= *qp_p) && (*qp_p <= '9')) || (('A' <= *qp_p) && (*qp_p <= 'F'))  || (('a' <= *qp_p) && (*qp_p <= 'f')) ) {
-    /* this is a valid hex char, if nan is unset */
-    if (nan) {
-      /* this is illegal */
-      *c = -2;
-      return initial_pos;
-    }
-    else {
-      hex[0] = *qp_p;
-      qp_p++;
-    };
-  }
-  else if (*qp_p == '\n') {
-    /* hit soft line break already, continue */
-    *c = -1;
-    return qp_p;
-  }
-  else {
-    /* illegal char here */
-    *c = -2;
-    return initial_pos;
+  /* Check for two hex digits and decode them */
+  if (isxdigit(*qp_p) && isxdigit(qp_p[1])) {
+    /* Do hex conversion */
+    if (isdigit(*qp_p)) {*c = *qp_p - '0';}
+    else {*c = toupper(*qp_p) - 'A' + 10;};
+    *c <<= 4;
+    if (isdigit(qp_p[1])) {*c |= qp_p[1] - '0';}
+    else {*c |= toupper(qp_p[1]) - 'A' + 10;};
+    return qp_p + 2;
   };
 
-  if ( (('0' <= *qp_p) && (*qp_p <= '9')) || (('A' <= *qp_p) && (*qp_p <= 'F')) || (('a' <= *qp_p) && (*qp_p <= 'f')) ) {
-    if (hex[0] > 0) {
-      hex[1] = *qp_p;
-      /* do hex conversion */
-      *c = mime_qp_hstr_i(hex);
-      qp_p++;
-      return qp_p;
-    }
-    else {
-      /* huh ? */
-      *c = -2;
-      return initial_pos;
-    };
-  }
-  else {
-    /* illegal char */
-    *c = -2;
-    return initial_pos;
+  /* tab or whitespace may follow just ignore it if it precedes \n */
+  while (*qp_p == '\t' || *qp_p == ' ' || *qp_p == '\r')
+    qp_p++;
+
+  if (*qp_p == '\n') {
+    /* hit soft line break */
+    *c = -1;
+    return qp_p;
   };
+
+  /* illegal char here */
+  *c = -2;
+  return initial_pos;
 }
 
 
-uschar *mime_parse_line(uschar *buffer, uschar *data, uschar *encoding, int *num_decoded) {
+/* just dump MIME part without any decoding */
+static int mime_decode_asis(FILE* in, FILE* out, uschar* boundary)
+{
+  int len, size = 0;
+  uschar buffer[MIME_MAX_LINE_LENGTH];
 
-  if (encoding == NULL) {
-    /* no encoding type at all */
-    NO_DECODING:
-    memcpy(data, buffer, Ustrlen(buffer));
-    data[(Ustrlen(buffer))] = 0;
-    *num_decoded = Ustrlen(data);
-    return data;
-  }
-  else if (Ustrcmp(encoding,"base64") == 0) {
-    uschar *p = buffer;
-    int offset = 0;
+  while(fgets(CS buffer, MIME_MAX_LINE_LENGTH, mime_stream) != NULL) {
+    if (boundary != NULL
+      && Ustrncmp(buffer, "--", 2) == 0
+      && Ustrncmp((buffer+2), boundary, Ustrlen(boundary)) == 0
+    )
+      break;
 
-    /* ----- BASE64 ---------------------------------------------------- */
-    /* NULL out '\r' and '\n' chars */
-    while (Ustrrchr(p,'\r') != NULL) {
-      *(Ustrrchr(p,'\r')) = '\0';
-    };
-    while (Ustrrchr(p,'\n') != NULL) {
-      *(Ustrrchr(p,'\n')) = '\0';
-    };
+    len = Ustrlen(buffer);
+    if (fwrite(buffer, 1, (size_t)len, out) < len)
+      return -1;
+    size += len;
+  } /* while */
+  return size;
+}
 
-    while (*(p+offset) != '\0') {
-      /* hit illegal char ? */
-      if (mime_b64[*(p+offset)] == 128) {
+
+/* decode base64 MIME part */
+static int mime_decode_base64(FILE* in, FILE* out, uschar* boundary)
+{
+  uschar ibuf[MIME_MAX_LINE_LENGTH], obuf[MIME_MAX_LINE_LENGTH];
+  uschar *ipos, *opos;
+  size_t len, size = 0;
+  int bytestate = 0;
+
+  opos = obuf;
+
+  while (Ufgets(ibuf, MIME_MAX_LINE_LENGTH, in) != NULL)
+  {
+    if (boundary != NULL
+      && Ustrncmp(ibuf, "--", 2) == 0
+      && Ustrncmp((ibuf+2), boundary, Ustrlen(boundary)) == 0
+    )
+      break;
+
+    for (ipos = ibuf ; *ipos != '\r' && *ipos != '\n' && *ipos != 0; ++ipos) {
+      /* skip padding */
+      if (*ipos == '=') {
+        ++bytestate;
+        continue;
+      }
+      /* skip bad characters */
+      if (mime_b64[*ipos] == 128) {
         mime_set_anomaly(MIME_ANOMALY_BROKEN_BASE64);
-        offset++;
+        continue;
       }
-      else {
-        *p = mime_b64[*(p+offset)];
-        p++;
-      };
-    };
-    *p = 255;
+      /* simple state-machine */
+      switch((bytestate++) & 3) {
+        case 0:
+          *opos = mime_b64[*ipos] << 2;
+           break;
+        case 1:
+          *opos |= mime_b64[*ipos] >> 4;
+          ++opos;
+          *opos = mime_b64[*ipos] << 4;
+          break;
+        case 2:
+          *opos |= mime_b64[*ipos] >> 2;
+          ++opos;
+          *opos = mime_b64[*ipos] << 6;
+          break;
+        case 3:
+          *opos |= mime_b64[*ipos];
+          ++opos;
+          break;
+      } /* switch */
+    } /* for */
+    /* something to write? */
+    len = opos - obuf;
+    if (len > 0) {
+      if (fwrite(obuf, 1, len, out) != len)
+        return -1; /* error */
+      size += len;
+      /* copy incomplete last byte to start of obuf, where we continue */
+      if ((bytestate & 3) != 0)
+        *obuf = *opos;
+      opos = obuf;
+    }
+  } /* while */
 
-    /* line is translated, start bit shifting */
-    p = buffer;
-    *num_decoded = 0;
-    while(*p != 255) {
-      uschar tmp_c;
-
-      /* byte 0 ---------------------- */
-      if (*(p+1) == 255) {
-        break;
-      }
-      data[(*num_decoded)] = *p;
-      data[(*num_decoded)] <<= 2;
-      tmp_c = *(p+1);
-      tmp_c >>= 4;
-      data[(*num_decoded)] |= tmp_c;
-      (*num_decoded)++;
-      p++;
-      /* byte 1 ---------------------- */
-      if (*(p+1) == 255) {
-        break;
-      }
-      data[(*num_decoded)] = *p;
-      data[(*num_decoded)] <<= 4;
-      tmp_c = *(p+1);
-      tmp_c >>= 2;
-      data[(*num_decoded)] |= tmp_c;
-      (*num_decoded)++;
-      p++;
-      /* byte 2 ---------------------- */
-      if (*(p+1) == 255) {
-        break;
-      }
-      data[(*num_decoded)] = *p;
-      data[(*num_decoded)] <<= 6;
-      data[(*num_decoded)] |= *(p+1);
-      (*num_decoded)++;
-      p+=2;
-
-    };
-    return data;
-    /* ----------------------------------------------------------------- */
+  /* write out last byte if it was incomplete */
+  if (bytestate & 3) {
+      if (fwrite(obuf, 1, 1, out) != 1)
+          return -1;
+      ++size;
   }
-  else if (Ustrcmp(encoding,"quoted-printable") == 0) {
-    uschar *p = buffer;
 
-    /* ----- QP -------------------------------------------------------- */
-    *num_decoded = 0;
-    while (*p != 0) {
-      if (*p == '=') {
+  return size;
+}
+
+
+/* decode quoted-printable MIME part */
+static int mime_decode_qp(FILE* in, FILE* out, uschar* boundary)
+{
+  uschar ibuf[MIME_MAX_LINE_LENGTH], obuf[MIME_MAX_LINE_LENGTH];
+  uschar *ipos, *opos;
+  size_t len, size = 0;
+
+  while (fgets(CS ibuf, MIME_MAX_LINE_LENGTH, in) != NULL)
+  {
+    if (boundary != NULL
+      && Ustrncmp(ibuf, "--", 2) == 0
+      && Ustrncmp((ibuf+2), boundary, Ustrlen(boundary)) == 0
+    )
+      break; /* todo: check for missing boundary */
+
+    ipos = ibuf;
+    opos = obuf;
+
+    while (*ipos != 0) {
+      if (*ipos == '=') {
         int decode_qp_result;
 
-        p = mime_decode_qp_char(p,&decode_qp_result);
+        ipos = mime_decode_qp_char(ipos, &decode_qp_result);
 
         if (decode_qp_result == -2) {
-          /* Error from decoder. p is unchanged. */
+          /* Error from decoder. ipos is unchanged. */
           mime_set_anomaly(MIME_ANOMALY_BROKEN_QP);
-          data[(*num_decoded)] = '=';
-          (*num_decoded)++;
-          p++;
+          *opos = '=';
+          ++opos;
+          ++ipos;
         }
         else if (decode_qp_result == -1) {
           break;
         }
         else if (decode_qp_result >= 0) {
-          data[(*num_decoded)] = decode_qp_result;
-          (*num_decoded)++;
-        };
+          *opos = decode_qp_result;
+          ++opos;
+        }
       }
       else {
-        data[(*num_decoded)] = *p;
-        (*num_decoded)++;
-        p++;
-      };
-    };
-    return data;
-    /* ----------------------------------------------------------------- */
+        *opos = *ipos;
+        ++opos;
+        ++ipos;
+      }
+    }
+    /* something to write? */
+    len = opos - obuf;
+    if (len > 0) {
+      if (fwrite(obuf, 1, len, out) != len)
+        return -1; /* error */
+      size += len;
+    }
   }
-  /* unknown encoding type, just dump as-is */
-  else goto NO_DECODING;
+  return size;
 }
 
 
@@ -278,10 +272,9 @@ int mime_decode(uschar **listptr) {
   uschar option_buffer[1024];
   uschar decode_path[1024];
   FILE *decode_file = NULL;
-  uschar *buffer = NULL;
-  uschar *decode_buffer = NULL;
   long f_pos = 0;
   unsigned int size_counter = 0;
+  int (*decode_function)(FILE*, FILE*, uschar*);
 
   if (mime_stream == NULL)
     return FAIL;
@@ -291,21 +284,6 @@ int mime_decode(uschar **listptr) {
   /* build default decode path (will exist since MBOX must be spooled up) */
   (void)string_format(decode_path,1024,"%s/scan/%s",spool_directory,message_id);
 
-  /* reserve a line and decoder buffer to work in */
-  buffer = (uschar *)malloc(MIME_MAX_LINE_LENGTH+1);
-  if (buffer == NULL) {
-    log_write(0, LOG_PANIC,
-                 "decode ACL condition: can't allocate %d bytes of memory.", MIME_MAX_LINE_LENGTH+1);
-    return DEFER;
-  };
-
-  decode_buffer = (uschar *)malloc(MIME_MAX_LINE_LENGTH+1);
-  if (decode_buffer == NULL) {
-    log_write(0, LOG_PANIC,
-                 "decode ACL condition: can't allocate %d bytes of memory.", MIME_MAX_LINE_LENGTH+1);
-    return DEFER;
-  };
-
   /* try to find 1st option */
   if ((option = string_nextinlist(&list, &sep,
                                   option_buffer,
@@ -346,50 +324,28 @@ int mime_decode(uschar **listptr) {
   if (decode_file == NULL)
     return DEFER;
 
-  /* read data linewise and dump it to the file,
-     while looking for the current boundary */
-  while(fgets(CS buffer, MIME_MAX_LINE_LENGTH, mime_stream) != NULL) {
-    uschar *decoded_line = NULL;
-    int decoded_line_length = 0;
+  /* decode according to mime type */
+  if (mime_content_transfer_encoding == NULL)
+    /* no encoding, dump as-is */
+    decode_function = mime_decode_asis;
+  else if (Ustrcmp(mime_content_transfer_encoding, "base64") == 0)
+    decode_function = mime_decode_base64;
+  else if (Ustrcmp(mime_content_transfer_encoding, "quoted-printable") == 0)
+    decode_function = mime_decode_qp;
+  else
+    /* unknown encoding type, just dump as-is */
+    decode_function = mime_decode_asis;
 
-    if (mime_current_boundary != NULL) {
-      /* boundary line must start with 2 dashes */
-      if (Ustrncmp(buffer,"--",2) == 0) {
-        if (Ustrncmp((buffer+2),mime_current_boundary,Ustrlen(mime_current_boundary)) == 0)
-          break;
-      };
-    };
-
-    decoded_line = mime_parse_line(buffer, decode_buffer, mime_content_transfer_encoding, &decoded_line_length);
-
-    /* write line to decode file */
-    if (fwrite(decoded_line, 1, decoded_line_length, decode_file) < decoded_line_length) {
-      /* error/short write */
-      clearerr(mime_stream);
-      fseek(mime_stream,f_pos,SEEK_SET);
-      return DEFER;
-    };
-    size_counter += decoded_line_length;
-
-    if (size_counter > 1023) {
-      if ((mime_content_size + (size_counter / 1024)) < 65535)
-        mime_content_size += (size_counter / 1024);
-      else
-        mime_content_size = 65535;
-      size_counter = (size_counter % 1024);
-    };
-
-  }
-
-  (void)fclose(decode_file);
+  size_counter = decode_function(mime_stream, decode_file, mime_current_boundary);
 
   clearerr(mime_stream);
-  fseek(mime_stream,f_pos,SEEK_SET);
+  fseek(mime_stream, f_pos, SEEK_SET);
 
-  /* round up remaining size bytes to one k */
-  if (size_counter) {
-    mime_content_size++;
-  };
+  if (fclose(decode_file) != 0 || size_counter < 0)
+    return DEFER;
+
+  /* round up to the next KiB */
+  mime_content_size = (size_counter + 1023) / 1024;
 
   return OK;
 }
