@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/routers/redirect.h,v 1.9 2007/01/08 10:50:20 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/routers/redirect.h,v 1.11 2009/11/16 19:50:38 nm4 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2007 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the redirect router */
@@ -28,6 +28,7 @@ typedef struct {
   uschar *sieve_subaddress;
   uschar *sieve_useraddress;
   uschar *sieve_vacation_directory;
+  uschar *sieve_enotify_mailto_owner;
   uschar *syntax_errors_text;
   uschar *syntax_errors_to;
   uschar *qualify_domain;
