@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/sieve.c,v 1.32 2007/10/11 12:44:30 michael Exp $ */
+/* $Cambridge: exim/exim-src/src/sieve.c,v 1.38 2009/11/05 19:40:51 nm4 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) Michael Haardt 2003-2007 */
+/* Copyright (c) Michael Haardt 2003-2008 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This code was contributed by Michael Haardt. */
@@ -71,6 +71,7 @@ struct Sieve
   int require_enotify;
   struct Notification *notified;
 #endif
+  uschar *enotify_mailto_owner;
 #ifdef SUBADDRESS
   int require_subaddress;
 #endif
@@ -611,8 +612,10 @@ return (match_prefix ? nl==0 : nl==0 && hl==0);
 
 /*
 Arguments:
-  needle      pattern to search ...
-  haystack    ... inside the haystack
+  needle          pattern to search ...
+  haystack        ... inside the haystack
+  ascii_caseless  ignore ASCII case
+  match_octet     match octets, not UTF-8 multi-octet characters
 
 Returns:      0               needle not found in haystack
               1               needle found
@@ -1002,7 +1005,7 @@ return quoted;
 *************************************************/
 
 /*
-According to RFC 3028, duplicate delivery to the same address must
+According to RFC 5228, duplicate delivery to the same address must
 not happen, so the list is first searched for the address.
 
 Arguments:
@@ -1937,7 +1940,7 @@ Grammar:
 Arguments:
   filter      points to the Sieve filter including its state
   n           total number of tests
-  true        number of passed tests
+  num_true    number of passed tests
   exec        Execute parsed statements
 
 Returns:      1                success
@@ -2945,7 +2948,7 @@ while (*filter->pc)
     struct String header;
     struct String subject;
     struct String body;
-    uschar *envelope_from,*envelope_to;
+    uschar *envelope_from;
     struct String auto_submitted_value;
     uschar *auto_submitted_def;
 
@@ -2968,8 +2971,7 @@ while (*filter->pc)
     subject.character=(uschar*)0;
     body.length=-1;
     body.character=(uschar*)0;
-    envelope_from=expand_string("$sender_address");
-    envelope_to=expand_string("$local_part_prefix$local_part$local_part_suffix@$domain");
+    envelope_from=(sender_address && sender_address[0]) ? expand_string(US"$local_part_prefix$local_part$local_part_suffix@$domain") : US "";
     for (;;)
       {
       if (parse_white(filter)==-1) return -1;
@@ -3020,94 +3022,94 @@ while (*filter->pc)
     if (parse_semicolon(filter)==-1) return -1;
     if (parse_mailto_uri(filter,method.character,&recipient,&header,&subject,&body)!=1)
       return -1;
-    if (message.length==-1) message=subject;
-    if (message.length==-1) expand_header(&message,&str_subject);
-    expand_header(&auto_submitted_value,&str_auto_submitted);
-    auto_submitted_def=expand_string(string_sprintf("${if def:header_auto-submitted {true}{false}}"));
-    if (auto_submitted_value.character == NULL || auto_submitted_def == NULL)
+    if (exec)
       {
-      filter->errmsg=CUS "header string expansion failed";
-      return -1;
-      }
-    if (Ustrcmp(auto_submitted_def,"true")!=0 || Ustrcmp(auto_submitted_value.character,"no")==0)
-      {
-      for (already=filter->notified; already; already=already->next)
+      if (message.length==-1) message=subject;
+      if (message.length==-1) expand_header(&message,&str_subject);
+      expand_header(&auto_submitted_value,&str_auto_submitted);
+      auto_submitted_def=expand_string(string_sprintf("${if def:header_auto-submitted {true}{false}}"));
+      if (auto_submitted_value.character == NULL || auto_submitted_def == NULL)
         {
-        if (already->method.length==method.length
-            && (method.length==-1 || strcmp(already->method.character,method.character)==0)
-            && already->importance.length==importance.length
-            && (importance.length==-1 || strcmp(already->importance.character,importance.character)==0)
-            && already->message.length==message.length
-            && (message.length==-1 || strcmp(already->message.character,message.character)==0))
-          break;
+        filter->errmsg=CUS "header string expansion failed";
+        return -1;
         }
-      if (already==(struct Notification*)0)
-        /* New notification, process it */
+        if (Ustrcmp(auto_submitted_def,"true")!=0 || Ustrcmp(auto_submitted_value.character,"no")==0)
         {
-        struct Notification *sent;
-        sent=store_get(sizeof(struct Notification));
-        sent->method=method;
-        sent->importance=importance;
-        sent->message=message;
-        sent->next=filter->notified;
-        filter->notified=sent;
-        if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
+        for (already=filter->notified; already; already=already->next)
           {
-          debug_printf("Notification to `%s': '%s'.\n",method.character,message.length!=-1 ? message.character : CUS "");
+          if (already->method.length==method.length
+              && (method.length==-1 || Ustrcmp(already->method.character,method.character)==0)
+              && already->importance.length==importance.length
+              && (importance.length==-1 || Ustrcmp(already->importance.character,importance.character)==0)
+              && already->message.length==message.length
+              && (message.length==-1 || Ustrcmp(already->message.character,message.character)==0))
+            break;
           }
-#ifndef COMPILE_SYNTAX_CHECKER
-        if (exec && filter_test == FTEST_NONE)
+        if (already==(struct Notification*)0)
+          /* New notification, process it */
           {
-          string_item *p;
-          header_line *h;
-          int pid,fd;
-
-          if ((pid = child_open_exim2(&fd,envelope_to,envelope_to))>=1)
+          struct Notification *sent;
+          sent=store_get(sizeof(struct Notification));
+          sent->method=method;
+          sent->importance=importance;
+          sent->message=message;
+          sent->next=filter->notified;
+          filter->notified=sent;
+  #ifndef COMPILE_SYNTAX_CHECKER
+          if (filter_test == FTEST_NONE)
             {
-            FILE *f;
-            uschar *buffer;
-            int buffer_capacity;
+            string_item *p;
+            int pid,fd;
 
-            f = fdopen(fd, "wb");
-            for (h = header_list; h != NULL; h = h->next)
-              if (h->type == htype_received) fprintf(f,"%s",h->text);
-            fprintf(f,"From: %s\n",from.length==-1 ? envelope_to : from.character);
-            for (p=recipient; p; p=p->next) fprintf(f,"To: %s\n",p->text);
-            fprintf(f,"Auto-submitted: sieve-notify\n");
-            if (header.length>0) fprintf(f,"%s",header.character);
-            if (message.length==-1)
+            if ((pid = child_open_exim2(&fd,envelope_from,envelope_from))>=1)
               {
-              message.character=US"Notification";
-              message.length=Ustrlen(message.character);
+              FILE *f;
+              uschar *buffer;
+              int buffer_capacity;
+
+              f = fdopen(fd, "wb");
+              fprintf(f,"From: %s\n",from.length==-1 ? expand_string(US"$local_part_prefix$local_part$local_part_suffix@$domain") : from.character);
+              for (p=recipient; p; p=p->next) fprintf(f,"To: %s\n",p->text);
+              fprintf(f,"Auto-Submitted: auto-notified; %s\n",filter->enotify_mailto_owner);
+              if (header.length>0) fprintf(f,"%s",header.character);
+              if (message.length==-1)
+                {
+                message.character=US"Notification";
+                message.length=Ustrlen(message.character);
+                }
+              /* Allocation is larger than neccessary, but enough even for split MIME words */
+              buffer_capacity=32+4*message.length;
+              buffer=store_get(buffer_capacity);
+              if (message.length!=-1) fprintf(f,"Subject: %s\n",parse_quote_2047(message.character, message.length, US"utf-8", buffer, buffer_capacity, TRUE));
+              fprintf(f,"\n");
+              if (body.length>0) fprintf(f,"%s\n",body.character);
+              fflush(f);
+              (void)fclose(f);
+              (void)child_close(pid, 0);
               }
-            /* Allocation is larger than neccessary, but enough even for split MIME words */
-            buffer_capacity=32+4*message.length;
-            buffer=store_get(buffer_capacity);
-            if (message.length!=-1) fprintf(f,"Subject: %s\n",parse_quote_2047(message.character, message.length, US"utf-8", buffer, buffer_capacity, TRUE));
-            fprintf(f,"\n");
-            if (body.length>0) fprintf(f,"%s\n",body.character);
-            fflush(f);
-            (void)fclose(f);
-            (void)child_close(pid, 0);
+            }
+          if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
+            {
+            debug_printf("Notification to `%s': '%s'.\n",method.character,message.length!=-1 ? message.character : CUS "");
+            }
+#endif
+          }
+        else
+          {
+          if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
+            {
+            debug_printf("Repeated notification to `%s' ignored.\n",method.character);
             }
           }
-#endif
         }
       else
         {
         if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
           {
-          debug_printf("Repeated notification to `%s' ignored.\n",method.character);
+          debug_printf("Ignoring notification, triggering message contains Auto-submitted: field.\n");
           }
         }
       }
-    else
-      {
-      if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
-        {
-        debug_printf("Ignoring notification, triggering message contains Auto-submitted: field.\n");
-        }
-      }
     }
 #endif
 #ifdef VACATION
@@ -3489,7 +3491,15 @@ while (parse_identifier(filter,CUS "require"))
     else if (eq_octet(check,&str_envelope_auth,0)) filter->require_envelope_auth=1;
 #endif
 #ifdef ENOTIFY
-    else if (eq_octet(check,&str_enotify,0)) filter->require_enotify=1;
+    else if (eq_octet(check,&str_enotify,0))
+      {
+      if (filter->enotify_mailto_owner == NULL)
+        {
+        filter->errmsg=CUS "enotify disabled";
+        return -1;
+        }
+        filter->require_enotify=1;
+      }
 #endif
 #ifdef SUBADDRESS
     else if (eq_octet(check,&str_subaddress,0)) filter->require_subaddress=1;
@@ -3537,7 +3547,8 @@ Arguments:
   filter      points to the entire file, read into store as a single string
   options     controls whether various special things are allowed, and requests
               special actions (not currently used)
-  sieve_vacation_directory  where to store vacation "once" files
+  vacation_directory    where to store vacation "once" files
+  enotify_mailto_owner  owner of mailto notifications
   useraddress string expression for :user part of address
   subaddress  string expression for :subaddress part of address
   generated   where to hang newly-generated addresses
@@ -3553,7 +3564,8 @@ Returns:      FF_DELIVERED     success, a significant action was taken
 
 int
 sieve_interpret(uschar *filter, int options, uschar *vacation_directory,
-  uschar *useraddress, uschar *subaddress, address_item **generated, uschar **error)
+  uschar *enotify_mailto_owner, uschar *useraddress, uschar *subaddress,
+  address_item **generated, uschar **error)
 {
 struct Sieve sieve;
 int r;
@@ -3579,6 +3591,20 @@ else
     }
   }
 
+if (enotify_mailto_owner == NULL)
+  sieve.enotify_mailto_owner = NULL;
+else
+  {
+  sieve.enotify_mailto_owner=expand_string(enotify_mailto_owner);
+  if (sieve.enotify_mailto_owner == NULL)
+    {
+    *error = string_sprintf("failed to expand \"%s\" "
+      "(sieve_enotify_mailto_owner): %s", enotify_mailto_owner,
+      expand_string_message);
+    return FF_ERROR;
+    }
+  }
+
 sieve.useraddress = useraddress == NULL ? CUS "$local_part_prefix$local_part$local_part_suffix" : useraddress;
 sieve.subaddress = subaddress;
 
